/* //////////////////////////////////////////////////////////
 * A Web Server using the ESP8266
 * that allows you to control a WS2812 RGB led light string/ring.
 *
 * Connect the DI pin on the WS2812 RGB LED light string/ring
 * to pin 12 on the ESP8266-12
 * or to pin 2 on the ESP8266-01
 * 
 * Change the led count in #define LED_COUNT 150 to match your led string/ring
 * Uncomment #define ESP8266_01 if using the ESP8266-01
 * 
 * After uploading this program to the ESP8266 module,
 * open the monitor and restart the ESP8266 module to 
 * see what IP address is assigned to this webserver,
 * then browse to this IP address.
 * 
 * or
 * 
 * Connect your WiFi to the default ssid 
 *     "WiFi setup ESP8266-xxxx"
 * or to the one you provided, then browse to the IP address
 *     1.2.3.4 
 * for the main page or
 *     1.2.3.4/setup
 * for the WiFi setup page
 * 
 * If the WiFi is not connected then you will have to go to
 * 1.2.3.4/setup to setup the ssid and password
 * 
 * If using the ESP8266-01 module, nothing will be printed
 * on the serial monitor because it conflicts with 
 * the status led on pin 1
 * 
 * Copyright 2016 Enoch Hwang
 */


//////////////////////////////////////////////////////////////
//// VCC voltage stuff
  ADC_MODE(ADC_VCC);


//////////////////////////////////////////////////////////////
//// status LED stuff
//#define ESP8266_01  // define this when using the ESP8266-01
#ifdef ESP8266_01
  #define statusLed 1     // for ESP8266-01
#else
  #define statusLed 2     // for ESP8266-12
#endif


//////////////////////////////////////////////////////////////
//// NTP clock stuff
#include "DateTime_RobotsForFun.h"


//////////////////////////////////////////////////////////////
//// WiFi stuff
#include "WiFi_RobotsForFun.h"


//////////////////////////////////////////////////////////////
#define NONBLOCKING_DELAY

// Do delay without using the delay() function
void nonblockingDelay(int wait) {
  unsigned long timesUp;
  timesUp = millis() + wait;
  while (millis() < timesUp) {
    flashActivityLed();
    server.handleClient();
    yield();  
  }
}


//////////////////////////////////////////////////////////////
//// WS2812 RGB LED stuff
#ifdef ESP8266_01
  #define DI_PIN 2
#else
  #define DI_PIN 12
#endif
#define LED_COUNT 150
#include <WS2812.h>
WS2812 leds = WS2812(LED_COUNT, DI_PIN, NEO_GRB + NEO_KHZ800);
// Need to define the NONBLOCKING_DELAY and function above 
// BEFORE the following #include "WS2812patterns.h"
// because the nonblockingDelay() callback function
// is used by all the routines in WS2812patterns.h
#define EXITNOW // to allow early termination of the RGB loops
#include "WS2812patterns.h"


// RGB lights modes
#define OFF 0
#define SOLID 1
#define WIPE 2
#define RAINBOW 3
#define BLOCKS 4
#define CHASE 5
#define CHASERAINBOW 6
#define RANDOM 7
#define DISCO 8
#define MOVEIN 9
#define FLASH 10
#define FADE 11
#define FADERAINBOW 12
#define CYCLONE 13
#define SINGLE 14
#define ALL 15
#define NONE 16
unsigned int RGBLightsMode = OFF;
byte red, green, blue;
int numberOfLeds, wait, times;

void checkLights() {
  exitNow = false; // to allow early termination of the RGB loops
  switch(RGBLightsMode) {
    case SOLID:
      int w;
      w = wait;
      if (wait == 0) w = 5000;
      allOn(leds, nonblockingDelay, Color(red,green,blue), w);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case WIPE:
      wipe(leds, nonblockingDelay, Color(red,green,blue), wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case RAINBOW:
      rainbow(leds, nonblockingDelay, wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case BLOCKS:
       movingBlocks(leds, nonblockingDelay, wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
       break;
    case CHASE:
      theaterChase(leds, nonblockingDelay, Color(red,green,blue), wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case CHASERAINBOW:
      theaterChaseRainbow(leds, nonblockingDelay, wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case RANDOM:
      randomFlash(leds, nonblockingDelay, numberOfLeds,wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case DISCO:
      discoLights(leds, nonblockingDelay, numberOfLeds,wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case MOVEIN:
      moveIn(leds, nonblockingDelay, numberOfLeds,wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case FLASH:
      flash(leds, nonblockingDelay, Color(red,green,blue),wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case FADE:
      fade(leds, nonblockingDelay, Color(red,green,blue),wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case FADERAINBOW:
      fadingRainbow(leds, nonblockingDelay, wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case CYCLONE:
      cyclone(leds, nonblockingDelay, Color(red,green,blue),wait,times);
      if (times > 0) RGBLightsMode = OFF; // turn off lights
      break;
    case SINGLE:
      single(leds, nonblockingDelay, Color(red,green,blue),wait,times);
      if (times > 0) RGBLightsMode = NONE; // do not turn off lights
      break;
    case ALL:
      wipe(leds, nonblockingDelay);
      if (exitNow) break;
      rainbow(leds, nonblockingDelay);
      if (exitNow) break;
      flash(leds, nonblockingDelay);
      if (exitNow) break;
      movingBlocks(leds, nonblockingDelay);
      if (exitNow) break;
      fade(leds, nonblockingDelay);
      if (exitNow) break;
      theaterChaseRainbow(leds, nonblockingDelay); 
      if (exitNow) break;
      cyclone(leds, nonblockingDelay);
      if (exitNow) break;
      randomFlash(leds, nonblockingDelay);
      if (exitNow) break;
      moveIn(leds, nonblockingDelay);
      if (exitNow) break;
      discoLights(leds, nonblockingDelay);
      break;
    case OFF:
      allOff(leds);
      RGBLightsMode = NONE;
      break;
    default:
      break;
  }
}


//////////////////////////////////////////////////////////////
//// Clock chime stuff
void checkClock() {
  static bool blueOn = false;
  int s = second();
  int m = minute();
  int h = hour();
  if ((h >= 18) && (h <= 21)) { // turn on blue color during these hours
    if (!blueOn) {
      blueOn = true;
      allOn(leds, nonblockingDelay, Color(0,0,255));
    }
  } else {
    if (blueOn) {
      blueOn = false;
      allOff(leds);
    }
  }
  if ((m == 0) && (s == 0)) {
    rainbow(leds, nonblockingDelay, 5,8);  // speed, number of times
    allOff(leds);
    blueOn = false;
  } else if ((m == 15) && (s == 0)) {
    cyclone(leds, nonblockingDelay, 0,0,10,1);  // color,wait,times,segments
    allOff(leds);
    blueOn = false;
  } else if ((m == 30) && (s == 0)) {
    cyclone(leds, nonblockingDelay, 0,0,10,1);  // color,wait,times,segments
    allOff(leds);
    blueOn = false;
  } else if ((m == 45) && (s == 0)) {
    cyclone(leds, nonblockingDelay, 0,0,10,1);  // color,wait,times,segments
    allOff(leds);
    blueOn = false;
  }
}


//////////////////////////////////////////////////////////////
//// HTML webpage 
void indexHTML() {
  String msg = "";

  msg += "<!DOCTYPE html>";
  msg += "<head>";
  msg += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">"; // keep fonts consistent between devices
//  msg += "<meta http-equiv=\"refresh\" content=\"1\">"; // auto refresh page every 1 second
  
  msg += "<title>Rainbow Lights</title>";
  msg += "  <style>";
  msg += "    body { background-color: #ffffff; font-family: Arial, Helvetica, Sans-Serif; Color: #000088; }";
  msg += "  </style>";
  msg += "</head>";
  
  msg += "<body>";
  msg += "<center><font size = \"3\"><b>RGB Rainbow Lights</b></font><br>";
  msg += "This is an ESP8266 demo webserver that allows you to control a RGB led ring/string.";
  msg += "<br>" + formattedTime();
  msg += "</center>";
  msg += "<hr>";

  // create the buttons
  msg += "<form style=\"font-size:14pt\">";
  msg += "Select color:";
  msg += "<br>&nbsp;&nbsp;<input type=\"color\" name=\"COLOR\" id=\"myrgb\" value=\"#xxxxxx\" />";
  msg += "<br><br>";
  msg += "Select pattern:";
  msg += "<table>";
  msg += "<tr><td>&nbsp;&nbsp;<input type=\"submit\" name=\"pattern\" value=\"SOLID\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"CHASE\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"WIPE\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;<input type=\"submit\" name=\"pattern\" value=\"BLOCKS\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"FLASH\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"DISCO\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;<input type=\"submit\" name=\"pattern\" value=\"RAINBOW\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"FADE\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"MOVEIN\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;<input type=\"submit\" name=\"pattern\" value=\"RANDOM\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"CYCLONE\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"SINGLE\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;<input type=\"submit\" name=\"pattern\" value=\"ALL\" style=\"font-size:14pt\" >";
  msg += "<td><input type=\"submit\" name=\"pattern\" value=\"OFF\" style=\"font-size:14pt\" >";
  msg += "</table>";
  msg += "<br>";
  msg += "Select options:";
  msg += "<table>";
//  msg += "<tr><td>&nbsp;&nbsp;Number of leds:<td><input type=\"number\" name=\"number\" min=\"1\" max=\"150\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;Delay:<td><input type=\"number\" name=\"wait\" min=\"1\" max=\"500\" style=\"font-size:14pt\" >";
  msg += "<tr><td>&nbsp;&nbsp;Number of times:<td><input type=\"number\" name=\"times\" min=\"1\" max=\"150\" style=\"font-size:14pt\" >";
  msg += "</table>";
  msg += "</form>";

  // wifi setup and info links
  msg += "<br><hr>IP address: ";
  if (wifiIsConnected) {
    msg += "<a href=\"/\">" + WiFi.localIP().toString() + "</a>";
    msg += "<br>Wifi setup and info: ";
    msg += "<a href=\"/setup\">" + WiFi.localIP().toString() + "/setup</a>";
  } else {
    msg += "<a href=\"http://1.2.3.4\">1.2.3.4</a>";
    msg += "<br>Wifi setup and info: ";
    msg += "<a href=\"http://1.2.3.4/setup\">1.2.3.4/setup</a>";
  }

  msg += "<br><br><hr>";  
  msg += "Go to <a href=\"http://www.RobotsForFun.com\">RobotsForFun.com</a> for more interesting projects.";

  msg += "</body>";
  msg += "</html>";

  // get the color input to update the webpage
  String color;
  if (server.argName(0).equals("COLOR")) {  // make sure the argument name is "COLOR"
    color = server.arg(0);
    msg.replace("#xxxxxx",color);
  }
  
  // send webpage to browser
  server.send ( 200, "text/html", msg);
  
  // get the selected color. convert color string in hex to decimal
  char s[4];
  color.substring(1,3).toCharArray(s,sizeof(s));  // convert String to c-string
  red = strtol(s,NULL,16);                        // convert hex string to value
  color.substring(3,5).toCharArray(s,sizeof(s));
  green = strtol(s,NULL,16);
  color.substring(5,7).toCharArray(s,sizeof(s));
  blue = strtol(s,NULL,16);
  
  // get url arguments and process the command that DO have time delays
  if (server.argName(2).equals("number")) {
    numberOfLeds = server.arg(2).toInt();
  }
  if (server.argName(2).equals("wait")) {
    wait = server.arg(2).toInt();
  }
  if (server.argName(3).equals("times")) {
    times = server.arg(3).toInt();
  }
  
  if (server.arg("pattern").equals("SOLID")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("solid ");
    RGBLightsMode = SOLID;
  } else if (server.arg("pattern").equals("WIPE")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("wipe ");
    RGBLightsMode = WIPE;
  } else if (server.arg("pattern").equals("RAINBOW")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("rainbow ");
    RGBLightsMode = RAINBOW;
  } else if (server.arg("pattern").equals("BLOCKS")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("blocks ");
    RGBLightsMode = BLOCKS;
  } else if (server.arg("pattern").equals("CHASE")) {
    exitNow = true;  // to allow early termination of the RGB loops
    if (red == 0 && green == 0 && blue == 0) {
      Serial.print("chaserainbow ");
      RGBLightsMode = CHASERAINBOW;
    } else {
      Serial.print("chase ");
      RGBLightsMode = CHASE;
    }
  } else if (server.arg("pattern").equals("RANDOM")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("random ");
    RGBLightsMode = RANDOM;
  } else if (server.arg("pattern").equals("DISCO")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("disco ");
    RGBLightsMode = DISCO;
  } else if (server.arg("pattern").equals("MOVEIN")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("movein ");
    RGBLightsMode = MOVEIN;
  } else if (server.arg("pattern").equals("FLASH")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("flash ");
    RGBLightsMode = FLASH;
  } else if (server.arg("pattern").equals("FADE")) {
    exitNow = true;  // to allow early termination of the RGB loops
    if (red == 0 && green == 0 && blue == 0) {
      Serial.print("faderainbow ");
      RGBLightsMode = FADERAINBOW;
    } else {
      Serial.print("fade ");
      RGBLightsMode = FADE;
    }
  } else if (server.arg("pattern").equals("CYCLONE")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("cyclone ");
    RGBLightsMode = CYCLONE;
  } else if (server.arg("pattern").equals("SINGLE")) {
    exitNow = true;  // to allow early termination of the RGB loops
    if (times == 0) times = 1;
    Serial.print("single ");
    RGBLightsMode = SINGLE;
  } else if (server.arg("pattern").equals("ALL")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("all ");
    RGBLightsMode = ALL;
  } else if (server.arg("pattern").equals("OFF")) {
    exitNow = true;  // to allow early termination of the RGB loops
    Serial.print("off ");
    RGBLightsMode = OFF;
  }  
}


void setup() {
  Serial.begin(115200);
  pinMode(statusLed, OUTPUT);
  randomSeed(analogRead(0));
  
  //// RGB LED stuff
  leds.begin();      // Call this to start up the LED strip.
  leds.clear();      // turns all LEDs off...
  leds.setBrightness(100);        // 0=off; 255=full bright
  leds.setPixelColor(0, 0,0,255); // turn on blue on pixel 0
  leds.show();
    
  //// WiFi stuff
  setupAP();  // setup access point using default ssid "WiFi setup ESP8266-xxxx" and no password
  char devicename[] = "ESP8266 Rainbow Lights"; // optional. change to whatever you want. Use "ping ESP8266" to test
  if (setupWiFi(devicename, false)) {  // no blocking
    leds.setPixelColor(0, 0,255,0); // turn on green on pixel 0
    leds.show();
    setClock(PST);
  } else {
    leds.setPixelColor(0, 255,0,0); // turn on red on pixel 0
    leds.show();
    // block for 3 minutes
    unsigned long timeout = millis() + 180000; // 3 minutes
    while(millis() < timeout) {
      flashActivityLed(3000,150,1);
      server.handleClient();
      yield();  // must have this to give time to handle wifi background stuff
    }
    ESP.restart();  // have not connected to the internet in 5 minutes so just reboot
  }
  
  // update systemStartTime and systemStartCount
  systemStartTime = now();
  EEPROM.get(EEPROM_ADDRESS_systemStartCount, systemStartCount);  // get systemStartCount from EEPROM
  systemStartCount++;
  EEPROM.put(EEPROM_ADDRESS_systemStartCount, systemStartCount);
  EEPROM.commit();

  delay(2000);
  allOff(leds);
}


void loop() {
  checkLights();
  checkClock();  
  
  if (APisOn && wifiIsConnected && (millis() > 600000)) turnOffAP(); // turn off AP after 10 minutes
  flashActivityLed();     // show system is alive
  server.handleClient();  // handle web client request
  //yield();  // yield() is called automatically at the end of loop() must have this to give time to handle wifi background stuff
}
